@@ -19,8 +19,11 @@
 
 .getStageResultsClassNames <- function() {
 	return(c("StageResultsMeans", 
-			"StageResultsRates",
-			"StageResultsSurvival"))
+		"StageResultsRates",
+		"StageResultsSurvival",
+		"StageResultsMultiArmMeans", 
+		"StageResultsMultiArmRates",
+		"StageResultsMultiArmSurvival"))
 }
 
 #' 
@@ -45,9 +48,6 @@
 #' @field weightsFisher The weights for Fisher's combination test.
 #' @field weightsInverseNormal The weights for inverse normal statistic.
 #' 
-#' @name StageResults_initialize
-#' Initializes the object.
-#' 
 #' @include class_core_parameter_set.R
 #' @include class_design.R
 #' @include class_analysis_dataset.R
@@ -65,11 +65,7 @@ StageResults <- setRefClass("StageResults",
 		.design = "TrialDesign",
 		.dataInput = "Dataset",
 		stages = "integer",
-		overallPValues = "numeric",
-		effectSizes = "numeric",
 		pValues = "numeric", 
-		combInverseNormal = "numeric", 
-		combFisher = "numeric", 
 		weightsFisher = "numeric", 
 		weightsInverseNormal = "numeric", 
 		thetaH0 = "numeric", 
@@ -96,24 +92,55 @@ StageResults <- setRefClass("StageResults",
 				.parameterNames <<- .getParameterNames(design)
 			}
 			.parameterFormatFunctions <<- C_PARAMETER_FORMAT_FUNCTIONS
+			
+			.setParameterType("pValues", ifelse(
+				.isMultiArm(), C_PARAM_NOT_APPLICABLE, C_PARAM_GENERATED))
+			.setParameterType("thetaH0", ifelse(
+				identical(thetaH0, C_THETA_H0_MEANS_DEFAULT), C_PARAM_DEFAULT_VALUE, C_PARAM_USER_DEFINED))
+			.setParameterType("direction", ifelse(
+				identical(direction, C_DIRECTION_UPPER), C_PARAM_DEFAULT_VALUE, C_PARAM_USER_DEFINED))
 		},
 		
 		getPlotSettings = function() {
 			return(.plotSettings)
 		},
 		
-		show = function() {
+		show = function(showType = 1, digits = NA_integer_) {
+			.show(showType = showType, digits = digits, consoleOutputEnabled = TRUE)
+		},
+		
+		.show = function(showType = 1, digits = NA_integer_, consoleOutputEnabled = TRUE) {
 			'Method for automatically printing stage results'
-			output <- .showParametersOfOneGroup(parameters = .getParametersToShow(), 
-				title = .toString(startWithUpperCase = TRUE), orderByParameterName = FALSE)
-			.showUnknownParameters()
-			invisible(output)
+			.resetCat()
+			if (showType == 2) {
+				.cat("Technical summary of the stage results object of class ",
+					methods::classLabel(class(.self)), ":\n\n", heading = 1,
+					consoleOutputEnabled = consoleOutputEnabled)
+				.showAllParameters(consoleOutputEnabled = consoleOutputEnabled)
+				.showParameterTypeDescription(consoleOutputEnabled = consoleOutputEnabled)
+			} else {		
+				.cat(.toString(startWithUpperCase = TRUE), ":\n\n", heading = 1,
+					consoleOutputEnabled = consoleOutputEnabled)
+				.showParametersOfOneGroup(.getUserDefinedParameters(), "User defined parameters",
+					orderByParameterName = FALSE, consoleOutputEnabled = consoleOutputEnabled)
+#				.showParametersOfOneGroup(.getDerivedParameters(), "Derived from user defined parameters",
+#					orderByParameterName = FALSE, consoleOutputEnabled = consoleOutputEnabled)
+				.showParametersOfOneGroup(.getDefaultParameters(), "Default parameters",
+					orderByParameterName = FALSE, consoleOutputEnabled = consoleOutputEnabled)
+				.showParametersOfOneGroup(.getGeneratedParameters(), "Output",
+					orderByParameterName = FALSE, consoleOutputEnabled = consoleOutputEnabled)
+				.showUnknownParameters(consoleOutputEnabled = consoleOutputEnabled)
+			}
 		},
 		
 		isDirectionUpper = function() {
 			return(direction == C_DIRECTION_UPPER)
 		},
 		
+		.isMultiArm = function() {
+			return(grepl("multi", tolower(class(.self))))
+		},
+		
 		.getParametersToShow = function() {
 			return(c("stages"))
 		},
@@ -124,14 +151,26 @@ StageResults <- setRefClass("StageResults",
 				return(paste(prefix, "means"))
 			}
 			
+			if (class(.self) == "StageResultsMultiArmMeans") {
+				return(paste(prefix, "multi-armed means"))
+			}
+			
 			if (class(.self) == "StageResultsRates") {
 				return(paste(prefix, "rates"))
 			}
 			
+			if (class(.self) == "StageResultsMultiArmRates") {
+				return(paste(prefix, "multi-armed rates"))
+			}
+			
 			if (class(.self) == "StageResultsSurvival") {
 				return(paste(prefix, "survival data"))
 			}
 			
+			if (class(.self) == "StageResultsMultiArmSurvival") {
+				return(paste(prefix, "multi-armed survival"))
+			}
+			
 			return("unknown stage results")
 		},
 		
@@ -164,8 +203,12 @@ StageResults <- setRefClass("StageResults",
 		},
 		
 		getNumberOfStages = function() {
+			if (.isMultiArm()) {
+				return(max(ncol(stats::na.omit(effectSizes)), 
+						ncol(stats::na.omit(separatePValues))))
+			}
 			return(max(length(stats::na.omit(effectSizes)), 
-					length(stats::na.omit(pValues))))
+				length(stats::na.omit(pValues))))
 		}
 	)
 )
@@ -192,9 +235,6 @@ StageResults <- setRefClass("StageResults",
 #' @field weightsFisher The weights for Fisher's combination test.
 #' @field weightsInverseNormal The weights for inverse normal statistic.
 #' 
-#' @name StageResultsMeans_initialize
-#' Initializes the object.
-#' 
 #' @include class_core_parameter_set.R
 #' @include class_design.R
 #' @include class_analysis_dataset.R
@@ -207,7 +247,11 @@ StageResults <- setRefClass("StageResults",
 StageResultsMeans <- setRefClass("StageResultsMeans",
 	contains = "StageResults",
 	fields = list(
+		combInverseNormal = "numeric", 
+		combFisher = "numeric", 
 		overallTestStatistics = "numeric", 
+		overallPValues = "numeric",
+		effectSizes = "numeric",
 		testStatistics = "numeric",
 		overallMeans = "numeric",
 		overallMeans1 = "numeric", 
@@ -226,6 +270,25 @@ StageResultsMeans <- setRefClass("StageResultsMeans",
 			callSuper(.design = design, .dataInput = dataInput, ...,  
 				equalVariances = equalVariances, normalApproximation = normalApproximation)
 			init(design = design, dataInput = dataInput)
+			
+			for (param in c(
+					"weightsFisher", 
+					"weightsInverseNormal", 
+					"combFisher", 
+					"combInverseNormal")) {
+				.setParameterType(param, C_PARAM_NOT_APPLICABLE)
+			}
+			
+			for (param in .getParametersToShow()) {
+				if (.getParameterType(param) == C_PARAM_TYPE_UNKNOWN) {
+					.setParameterType(param, C_PARAM_GENERATED)
+				}
+			}
+			
+			.setParameterType("equalVariances", ifelse(
+					identical(equalVariances, TRUE), C_PARAM_DEFAULT_VALUE, C_PARAM_USER_DEFINED))
+			.setParameterType("normalApproximation", ifelse(
+					identical(normalApproximation, FALSE), C_PARAM_DEFAULT_VALUE, C_PARAM_USER_DEFINED))
 		},
 		.getParametersToShow = function() {
 			parametersToShow <- c(
@@ -283,6 +346,94 @@ StageResultsMeans <- setRefClass("StageResultsMeans",
 	)
 )
 
+#' @include class_core_parameter_set.R
+#' @include class_design.R
+#' @include class_analysis_dataset.R
+#' @include f_core_constants.R
+#' 
+#' @keywords internal
+#' 
+#' @importFrom methods new
+#'
+StageResultsMultiArmMeans <- setRefClass("StageResultsMultiArmMeans",
+	contains = "StageResults",
+	fields = list(
+		stage = "integer",
+		combInverseNormal = "matrix", 
+		combFisher = "matrix", 
+		overallTestStatistics = "matrix", 
+		overallStDevs = "matrix",
+		overallPValues = "matrix",
+		testStatistics = "matrix",
+		separatePValues = "matrix",
+		effectSizes = "matrix",
+		singleStepAdjustedPValues = "matrix",
+		intersectionTest = "character",
+		varianceOption = "character",
+		normalApproximation = "logical",
+		directionUpper = "logical" 
+	),
+	methods = list(
+		initialize = function(design, dataInput, ..., varianceOption = C_VARIANCE_OPTION_DEFAULT, 
+				normalApproximation = FALSE) {
+			callSuper(.design = design, .dataInput = dataInput, ...,  
+				varianceOption = varianceOption, normalApproximation = normalApproximation)
+			init(design = design, dataInput = dataInput)
+			
+			for (param in c("singleStepAdjustedPValues",
+					"weightsFisher", 
+					"weightsInverseNormal", 
+					"combFisher", 
+					"combInverseNormal")) {
+				.setParameterType(param, C_PARAM_NOT_APPLICABLE)
+			}
+			
+			for (param in .getParametersToShow()) {
+				if (.getParameterType(param) == C_PARAM_TYPE_UNKNOWN) {
+					.setParameterType(param, C_PARAM_GENERATED)
+				}
+			}
+			
+			.setParameterType("varianceOption", ifelse(
+				identical(varianceOption, C_VARIANCE_OPTION_DEFAULT), C_PARAM_DEFAULT_VALUE, C_PARAM_USER_DEFINED))
+			.setParameterType("normalApproximation", ifelse(
+				identical(normalApproximation, FALSE), C_PARAM_DEFAULT_VALUE, C_PARAM_USER_DEFINED))
+			.setParameterType("directionUpper", ifelse(
+				identical(directionUpper, C_DIRECTION_UPPER_DEFAULT), C_PARAM_DEFAULT_VALUE, C_PARAM_USER_DEFINED))
+		},
+		.getParametersToShow = function() {
+			parametersToShow <- c(
+				"stages",
+				"thetaH0",
+				"direction",
+				"normalApproximation",
+				#"directionUpper",
+				"varianceOption",
+				"overallTestStatistics", 
+				"overallPValues",
+				"overallStDevs",
+				"testStatistics", 
+				"separatePValues", 
+				"effectSizes",
+				"singleStepAdjustedPValues"
+			)
+			if (.isTrialDesignInverseNormal(.design)) {
+				parametersToShow <- c(parametersToShow,			
+					"combInverseNormal", 
+					"weightsInverseNormal"
+				)
+			}
+			else if (.isTrialDesignFisher(.design)) {
+				parametersToShow <- c(parametersToShow,
+					"combFisher", 
+					"weightsFisher"
+				)
+			}
+			return(parametersToShow)
+		}
+	)
+)
+
 #' 
 #' @name StageResultsRates
 #' 
@@ -305,9 +456,6 @@ StageResultsMeans <- setRefClass("StageResultsMeans",
 #' @field weightsFisher The weights for Fisher's combination test.
 #' @field weightsInverseNormal The weights for inverse normal statistic.
 #' 
-#' @name StageResultsRates_initialize
-#' Initializes the object.
-#' 
 #' @include class_core_parameter_set.R
 #' @include class_design.R
 #' @include class_analysis_dataset.R
@@ -320,7 +468,11 @@ StageResultsMeans <- setRefClass("StageResultsMeans",
 StageResultsRates <- setRefClass("StageResultsRates",
 	contains = "StageResults",
 	fields = list(
+		combInverseNormal = "numeric", 
+		combFisher = "numeric", 
 		overallTestStatistics = "numeric", 
+		overallPValues = "numeric",
+		effectSizes = "numeric",
 		testStatistics = "numeric",
 		overallEvents = "numeric",
 		overallEvents1 = "numeric", 
@@ -335,6 +487,23 @@ StageResultsRates <- setRefClass("StageResultsRates",
 			callSuper(.design = design, .dataInput = dataInput, ...,  
 				normalApproximation = normalApproximation)
 			init(design = design, dataInput = dataInput)
+			
+			for (param in c(
+					"weightsFisher", 
+					"weightsInverseNormal", 
+					"combFisher", 
+					"combInverseNormal")) {
+				.setParameterType(param, C_PARAM_NOT_APPLICABLE)
+			}
+			
+			for (param in .getParametersToShow()) {
+				if (.getParameterType(param) == C_PARAM_TYPE_UNKNOWN) {
+					.setParameterType(param, C_PARAM_GENERATED)
+				}
+			}
+			
+			.setParameterType("normalApproximation", ifelse(
+					identical(normalApproximation, TRUE), C_PARAM_DEFAULT_VALUE, C_PARAM_USER_DEFINED))
 		},
 		.getParametersToShow = function() {
 			parametersToShow <- c(
@@ -384,6 +553,91 @@ StageResultsRates <- setRefClass("StageResultsRates",
 	)
 )
 
+#' @include class_core_parameter_set.R
+#' @include class_design.R
+#' @include class_analysis_dataset.R
+#' @include f_core_constants.R
+#' 
+#' @keywords internal
+#' 
+#' @importFrom methods new
+#'
+StageResultsMultiArmRates <- setRefClass("StageResultsMultiArmRates",
+		contains = "StageResults",
+		fields = list(
+				stage = "integer",
+				piControl = "matrix",
+				piTreatments = "matrix",
+				combInverseNormal = "matrix", 
+				combFisher = "matrix", 
+				overallTestStatistics = "matrix", 
+				overallPValues = "matrix",
+				testStatistics = "matrix",
+				separatePValues = "matrix",
+				singleStepAdjustedPValues = "matrix",
+				intersectionTest = "character",
+				normalApproximation = "logical",
+				directionUpper = "logical" 
+		),
+		methods = list(
+				initialize = function(design, dataInput, ..., 
+						normalApproximation = FALSE) {
+					callSuper(.design = design, .dataInput = dataInput, ...,  
+							normalApproximation = normalApproximation)
+					init(design = design, dataInput = dataInput)
+					
+					for (param in c("singleStepAdjustedPValues",
+							"weightsFisher", 
+							"weightsInverseNormal", 
+							"combFisher", 
+							"combInverseNormal")) {
+						.setParameterType(param, C_PARAM_NOT_APPLICABLE)
+					}
+					
+					for (param in .getParametersToShow()) {
+						if (.getParameterType(param) == C_PARAM_TYPE_UNKNOWN) {
+							.setParameterType(param, C_PARAM_GENERATED)
+						}
+					}
+					
+					.setParameterType("normalApproximation", ifelse(
+									identical(normalApproximation, FALSE), C_PARAM_DEFAULT_VALUE, C_PARAM_USER_DEFINED))
+					.setParameterType("directionUpper", ifelse(
+									identical(directionUpper, C_DIRECTION_UPPER_DEFAULT), C_PARAM_DEFAULT_VALUE, C_PARAM_USER_DEFINED))
+				},
+				.getParametersToShow = function() {
+					parametersToShow <- c(
+							"stages",
+							"thetaH0",
+							"direction",
+							"normalApproximation",
+							#"directionUpper",
+							"piControl",
+							"piTreatments",
+							"overallTestStatistics", 
+							"overallPValues",
+							"overallStDevs",
+							"testStatistics", 
+							"separatePValues", 
+							"singleStepAdjustedPValues"
+					)
+					if (.isTrialDesignInverseNormal(.design)) {
+						parametersToShow <- c(parametersToShow,			
+								"combInverseNormal", 
+								"weightsInverseNormal"
+						)
+					}
+					else if (.isTrialDesignFisher(.design)) {
+						parametersToShow <- c(parametersToShow,
+								"combFisher", 
+								"weightsFisher"
+						)
+					}
+					return(parametersToShow)
+				}
+		)
+)
+
 #' 
 #' @name StageResultsSurvival
 #' 
@@ -406,9 +660,6 @@ StageResultsRates <- setRefClass("StageResultsRates",
 #' @field weightsFisher The weights for Fisher's combination test.
 #' @field weightsInverseNormal The weights for inverse normal statistic.
 #' 
-#' @name StageResultsSurvival_initialize
-#' Initializes the object.
-#' 
 #' @include class_core_parameter_set.R
 #' @include class_design.R
 #' @include class_analysis_dataset.R
@@ -421,6 +672,10 @@ StageResultsRates <- setRefClass("StageResultsRates",
 StageResultsSurvival <- setRefClass("StageResultsSurvival",
 	contains = "StageResults",
 	fields = list(
+		combInverseNormal = "numeric", 
+		combFisher = "numeric", 
+		overallPValues = "numeric",
+		effectSizes = "numeric",
 		overallLogRanks = "numeric",
 		overallEvents = "numeric", 
 		overallAllocationRatios = "numeric",
@@ -432,6 +687,20 @@ StageResultsSurvival <- setRefClass("StageResultsSurvival",
 		initialize = function(design, dataInput, ...) {
 			callSuper(.design = design, .dataInput = dataInput, ...)
 			init(design = design, dataInput = dataInput)
+			
+			for (param in c(
+					"weightsFisher", 
+					"weightsInverseNormal", 
+					"combFisher", 
+					"combInverseNormal")) {
+				.setParameterType(param, C_PARAM_NOT_APPLICABLE)
+			}
+			
+			for (param in .getParametersToShow()) {
+				if (.getParameterType(param) == C_PARAM_TYPE_UNKNOWN) {
+					.setParameterType(param, C_PARAM_GENERATED)
+				}
+			}
 		},
 		.getParametersToShow = function() {
 			parametersToShow <- c(
@@ -468,6 +737,85 @@ StageResultsSurvival <- setRefClass("StageResultsSurvival",
 	)
 )
 
+
+#' @include class_core_parameter_set.R
+#' @include class_design.R
+#' @include class_analysis_dataset.R
+#' @include f_core_constants.R
+#' 
+#' @keywords internal
+#' 
+#' @importFrom methods new
+#'
+StageResultsMultiArmSurvival <- setRefClass("StageResultsMultiArmSurvival",
+		contains = "StageResults",
+		fields = list(
+			stage = "integer",
+			combInverseNormal = "matrix", 
+			combFisher = "matrix", 
+			overallTestStatistics = "matrix", 
+			overallPValues = "matrix",
+			testStatistics = "matrix",
+			separatePValues = "matrix",
+			effectSizes = "matrix",
+			singleStepAdjustedPValues = "matrix",
+			intersectionTest = "character",
+			directionUpper = "logical" 
+		),
+		methods = list(
+				initialize = function(design, dataInput, ..., 
+						normalApproximation = FALSE) {
+					callSuper(.design = design, .dataInput = dataInput, ...)
+					init(design = design, dataInput = dataInput)
+					
+					for (param in c("singleStepAdjustedPValues",
+							"weightsFisher", 
+							"weightsInverseNormal", 
+							"combFisher", 
+							"combInverseNormal")) {
+						.setParameterType(param, C_PARAM_NOT_APPLICABLE)
+					}
+					
+					for (param in .getParametersToShow()) {
+						if (.getParameterType(param) == C_PARAM_TYPE_UNKNOWN) {
+							.setParameterType(param, C_PARAM_GENERATED)
+						}
+					}
+					
+					.setParameterType("directionUpper", ifelse(
+									identical(directionUpper, C_DIRECTION_UPPER_DEFAULT), C_PARAM_DEFAULT_VALUE, C_PARAM_USER_DEFINED))
+				},
+				.getParametersToShow = function() {
+					parametersToShow <- c(
+							"stages",
+							"thetaH0",
+							"direction",
+							#"directionUpper",
+							"overallTestStatistics", 
+							"overallPValues",
+							"testStatistics", 
+							"separatePValues", 
+							"effectSizes",
+							"singleStepAdjustedPValues"
+					)
+					if (.isTrialDesignInverseNormal(.design)) {
+						parametersToShow <- c(parametersToShow,			
+								"combInverseNormal", 
+								"weightsInverseNormal"
+						)
+					}
+					else if (.isTrialDesignFisher(.design)) {
+						parametersToShow <- c(parametersToShow,
+								"combFisher", 
+								"weightsFisher"
+						)
+					}
+					return(parametersToShow)
+				}
+		)
+)
+
+
 #'
 #' @name StageResults_names
 #' 
@@ -505,7 +853,8 @@ names.StageResults <- function(x) {
 #' @keywords internal
 #'  
 as.data.frame.StageResults <- function(x, row.names = NULL, 
-		optional = FALSE, niceColumnNamesEnabled = TRUE, includeAllParameters = FALSE, type = 1, ...) {
+		optional = FALSE, niceColumnNamesEnabled = FALSE, 
+		includeAllParameters = FALSE, type = 1, ...) {
 	
 	if (type == 1) {
 		parametersToShow <- x$.getParametersToShow()
@@ -541,62 +890,8 @@ as.data.frame.StageResults <- function(x, row.names = NULL,
 }
 
 #'
-#' @name StageResults_summary
-#' 
 #' @title
-#' Stage Results Summary
-#'
-#' @description
-#' Displays a summary of \code{StageResults} object.
-#' 
-#' @details
-#' Summarizes the parameters and results of stage results.
-#' 
-#' @export
-#' 
-#' @keywords internal
-#' 
-summary.StageResults <- function(object, ...) {
-	cat("This output summarizes the ", object$.toString(), ".\n\n", sep = "")
-	object$show()
-	cat("\n")
-	print.StageResults(object, ...) 
-}
-
-#'
-#' @name StageResults_print
-#' 
-#' @title
-#' Print Stage Results
-#'
-#' @description
-#' \code{print} prints its \code{StageResults} argument and returns it invisibly (via \code{invisible(x)}). 
-#' 
-#' @details
-#' Prints the parameters and values of stage results.
-#'
-#' @export
-#' 
-#' @keywords internal
-#' 
-print.StageResults <- function(x, ...) {
-	cat(x$.toString(startWithUpperCase = TRUE), "table:\n")
-	parametersToShow <- x$.getParametersToShow()
-	parametersToShow <- parametersToShow[parametersToShow != "stages"]
-	for (parameter in parametersToShow) {
-		if (length(x[[parameter]]) == 1) {
-			parametersToShow <- parametersToShow[parametersToShow != parameter]
-		}
-	}
-	x$.printAsDataFrame(parameterNames = parametersToShow)
-	invisible(x)
-}
-
-#'
-#' @name StageResults_plot
-#' 
-#' @title
-#' Stage Results Plotting - Conditional Power Plot
+#' Stage Results Plotting
 #' 
 #' @description
 #' Plots the conditional power together with the likelihood function. 
@@ -605,9 +900,10 @@ print.StageResults <- function(x, ...) {
 #' @param y Not available for this kind of plot (is only defined to be compatible to the generic plot function).
 #' @param stage The stage number (optional). Default: total number of existing stages in the data input
 #'        used to create the stage results.
-#' @param nPlanned The sample size planned for the subsequent stages. 
-#'        It should be a vector with length equal to the remaining stages and is the 
-#'        overall sample size in the two treatment groups if two groups are considered.   
+#' @param nPlanned The additional (i.e. "new" and not cumulative) sample size planned for each of the subsequent stages. 
+#'        The argument should be a vector with length equal to the number of remaining stages and contain 
+#'        the combined sample size from both treatment groups if two groups are considered. For survival outcomes, 
+#'        it should contain the planned number of additional events. 
 #' @param allocationRatioPlanned The allocation ratio for two treatment groups planned for 
 #'        the subsequent stages, the default value is 1. 
 #' @param main The main title.
@@ -615,6 +911,9 @@ print.StageResults <- function(x, ...) {
 #' @param ylab The y-axis label.
 #' @param legendTitle The legend title.
 #' @param palette The palette, default is \code{"Set1"}.
+#' @param showSource If \code{TRUE}, the parameter names of the object will 
+#'        be printed which were used to create the plot; that may be, e.g., 
+#'        useful to check the values or to create own plots with \code{\link[graphics]{plot}}.
 #' @param legendPosition The position of the legend. 
 #' By default (\code{NA_integer_}) the algorithm tries to find a suitable position. 
 #' Choose one of the following values to specify the position manually:
@@ -649,6 +948,9 @@ print.StageResults <- function(x, ...) {
 #' Generic function to plot all kinds of stage results.
 #' The conditional power is calculated only if effect size and sample size is specified. 
 #' 
+#' @return 
+#' A \code{ggplot2} object.
+#' 
 #' @export
 #' 
 #' @examples 
@@ -667,12 +969,12 @@ print.StageResults <- function(x, ...) {
 #' 
 #' if (require(ggplot2)) plot(stageResults, nPlanned = c(30), thetaRange = c(0, 100))
 #'
-plot.StageResults <- function(x, y, ...,
+plot.StageResults <- function(x, y, ..., type = 1L,
 		nPlanned, stage = x$getNumberOfStages(),
-		allocationRatioPlanned = NA_real_,
+		allocationRatioPlanned = C_ALLOCATION_RATIO_DEFAULT,
 		main = NA_character_, xlab = NA_character_, ylab = NA_character_,
 		legendTitle = NA_character_, palette = "Set1", legendPosition = NA_integer_, 
-		type = 1) {
+		showSource = FALSE) {
 	
 	.assertGgplotIsInstalled()
 	.assertIsStageResults(x)
@@ -680,25 +982,52 @@ plot.StageResults <- function(x, y, ...,
 	
 	plotData <- .getConditionalPowerPlot(stageResults = x, nPlanned = nPlanned, stage = stage,
 		allocationRatioPlanned = allocationRatioPlanned, ...)
-	
-	classicPlotEnabled <- FALSE 
-	if (classicPlotEnabled) {
-		graphics::plot(plotData$xValues, plotData$condPowerValues, type = 'l', lwd = 2, col = "black", lty = 1, 
-			ylim = c(0,1), xlab = plotData$xlab, ylab = plotData$ylab,
-			main = plotData$main, sub = plotData$sub)
-		graphics::lines(plotData$xValues, plotData$likelihoodValues,  type = 'l', lty = 3, lwd = 2, col = "darkgreen") 
-		return()
-	}
-
+		
 	yParameterName1 <- "Conditional power"
 	yParameterName2 <- "Likelihood"
 	
-	data <- data.frame(
-		xValues = c(plotData$xValues, plotData$xValues),
-		yValues = c(plotData$condPowerValues, plotData$likelihoodValues),
-		categories = c(rep(yParameterName1, length(plotData$xValues)), 
-			rep(yParameterName2, length(plotData$xValues)))
-	)
+	if (x$.isMultiArm()) {
+		numberOfTreatments <- nrow(x$testStatistics)
+		
+		treatmentArmsToShow <- as.integer(list(...)[["treatmentArms"]])
+		if (is.null(treatmentArmsToShow) || length(treatmentArmsToShow) == 0 || 
+				is.na(treatmentArmsToShow) || !is.numeric(treatmentArmsToShow)) {
+			treatmentArmsToShow <- 1L:as.integer(numberOfTreatments)
+		}
+		data <- data.frame(
+			xValues = numeric(0),
+			yValues = numeric(0),
+			categories = character(0)
+		)
+		for (treatmentArm in treatmentArmsToShow) {
+			legend1 <- ifelse(length(treatmentArmsToShow) == 1, yParameterName1,
+				paste0(yParameterName1, " (", treatmentArm, ")"))
+			legend2 <- ifelse(length(treatmentArmsToShow) == 1, yParameterName2,
+				paste0(yParameterName2, " (", treatmentArm, ")"))
+			if (all(is.na(plotData$condPowerValues[treatmentArm, ]))) {
+				data <- rbind(data, data.frame(
+					xValues = plotData$xValues,
+					yValues = plotData$likelihoodValues[treatmentArm, ],
+					categories = rep(legend2, length(plotData$xValues))
+				)) 
+			} else {
+				data <- rbind(data, data.frame(
+					xValues = c(plotData$xValues, plotData$xValues),
+					yValues = c(plotData$condPowerValues[treatmentArm, ], 
+						plotData$likelihoodValues[treatmentArm, ]),
+					categories = c(rep(legend1, length(plotData$xValues)), 
+						rep(legend2, length(plotData$xValues)))
+				)) 
+			}
+		}
+	} else {
+		data <- data.frame(
+			xValues = c(plotData$xValues, plotData$xValues),
+			yValues = c(plotData$condPowerValues, plotData$likelihoodValues),
+			categories = c(rep(yParameterName1, length(plotData$xValues)), 
+				rep(yParameterName2, length(plotData$xValues)))
+		)
+	}
 	
 	p <- ggplot2::ggplot(data, ggplot2::aes(x = data$xValues, y = data$yValues, 
 		colour = factor(data$categories)))
